package com.hz.pm.api.projectlib.manage;

import com.hz.pm.api.common.util.ThreadPoolUtil;
import com.hz.pm.api.projectdeclared.manage.*;
import com.hz.pm.api.projectdeclared.model.req.PurchaseAdaptionListReq;
import com.hz.pm.api.projectdeclared.model.vo.*;
import com.hz.pm.api.projectlib.model.req.DeclaredProjectListReq;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.AmountApprovalProgressStatisticsVO;
import com.hz.pm.api.projectlib.model.vo.AnnualPlanProgressStatisticsVO;
import com.hz.pm.api.projectlib.model.vo.DeclaredRecordProgressStatVO;
import com.hz.pm.api.projectlib.model.vo.ProjectManageTodoCountVO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.CompletableFuture;
import java.util.function.Function;

/**
 * <p>
 * ProjectTodoCountManage
 * </p>
 *
 * @author WendyYang
 * @since 10:51 2024/10/31
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class ProjectManageTodoCountManage {

    private final DeclaredProjectManage declaredProjectManage;
    private final AnnualPlanLibManage annualPlanLibManage;
    private final AmountApprovalManage amountApprovalManage;
    private final DeclaredRecordManage declaredRecordManage;
    private final PurchaseManage purchaseManage;
    private final ConstructionManage constructionManage;
    private final FinalAcceptanceManage finalAcceptanceManage;
    private final ProjectReviewManage projectReviewManage;

    public ProjectManageTodoCountVO allTodoCount() {
        ProjectManageTodoCountVO todoCount = new ProjectManageTodoCountVO();
        List<CompletableFuture<Void>> futures = new ArrayList<>();
        futures.add(CompletableFuture.runAsync(() -> {
            // 项目申报
            DeclaredProgressStatisticsVO declaredStat = declaredProjectManage.declaredProgressStatistics(new ProjectListReq());
            todoCount.setProjectDeclare(sum(declaredStat, DeclaredProgressStatisticsVO::getTodoCount,
                    DeclaredProgressStatisticsVO::getFailedCount));
            // 专家评审
            ReviewProgressStatisticsVO expertReviewStat = projectReviewManage.expertReviewProjectProgressStatistics(new ProjectListReq());
            todoCount.setExpertReview(sum(expertReviewStat, ReviewProgressStatisticsVO::getTodoCount));
            // 项目复核
            ReviewProgressStatisticsVO projectReviewStat = projectReviewManage.reviewProjectProgressStatistics(new ProjectListReq());
            todoCount.setProjectReview(sum(projectReviewStat, ReviewProgressStatisticsVO::getTodoCount,
                    ReviewProgressStatisticsVO::getFailedCount));
        }, ThreadPoolUtil.REQUEST));
        futures.add(CompletableFuture.runAsync(() -> {
            // 年度计划
            AnnualPlanProgressStatisticsVO annualStat = annualPlanLibManage.annualProgressStatistics(new ProjectListReq());
            todoCount.setAnnualPlan(sum(annualStat, AnnualPlanProgressStatisticsVO::getNotOpenAnnualPlanCount));
            // 资金下达
            AmountApprovalProgressStatisticsVO amountApprovalStat = amountApprovalManage.progressStatistics(new ProjectListReq());
            todoCount.setAmountApproval(sum(amountApprovalStat, AmountApprovalProgressStatisticsVO::getUnApprovalCount));
            // 立项备案
            DeclaredRecordProgressStatVO recordStat = declaredRecordManage.declaredRecordProgressStatistics(new DeclaredProjectListReq());
            todoCount.setProjectRecord(sum(recordStat, DeclaredRecordProgressStatVO::getTodoCount,
                    DeclaredRecordProgressStatVO::getFailedCount));
        }, ThreadPoolUtil.REQUEST));
        futures.add(CompletableFuture.runAsync(() -> {
            // 招标采购
            PurchaseProgressStatVO purchaseStat = purchaseManage.purchaseProgressStatistics(new ProjectListReq());
            todoCount.setProjectPurchase(sum(purchaseStat, PurchaseProgressStatVO::getOnPurchaseCount));
            // 适配改造
            AdaptionProgressStatVO adaptionStat = purchaseManage.adaptionProgressStatistics(new PurchaseAdaptionListReq());
            todoCount.setProjectAdaption(sum(adaptionStat, AdaptionProgressStatVO::getWithoutOperationCount,
                    AdaptionProgressStatVO::getWithoutAdaptionInfo,
                    AdaptionProgressStatVO::getWithoutTestValidCount,
                    AdaptionProgressStatVO::getWithoutOrgConfirmCount,
                    AdaptionProgressStatVO::getWithoutSelfTestCount));
            // 项目初验
            FirstAcceptProgressStatisticsVO firstAcceptStat = constructionManage.firstAcceptProgressStatistics(new ProjectListReq());
            todoCount.setFirstAccept(sum(firstAcceptStat, FirstAcceptProgressStatisticsVO::getTodoCount));
            // 项目终验
            FinalAcceptProgressStatisticsVO finalAcceptStat = finalAcceptanceManage.finalAcceptProgressStatistics(new ProjectListReq());
            todoCount.setFinalAccept(sum(finalAcceptStat, FinalAcceptProgressStatisticsVO::getTodoCount,
                    FinalAcceptProgressStatisticsVO::getFailedCount));
        }, ThreadPoolUtil.REQUEST));
        CompletableFuture.allOf(futures.toArray(new CompletableFuture[0])).join();
        return todoCount;
    }

    @SafeVarargs
    private final <T> int sum(T obj, Function<T, Number>... function) {
        if (obj == null || function == null || function.length == 0) {
            return 0;
        }
        return Arrays.stream(function)
                .map(w -> w.apply(obj))
                .filter(Objects::nonNull)
                .mapToInt(Number::intValue)
                .sum();
    }

}
